@@ -1,7 +1,7 @@
 /* Emulation for poll(2)
    Contributed by Paolo Bonzini.
 
-   Copyright 2001, 2002, 2003, 2006 Free Software Foundation, Inc.
+   Copyright 2001, 2002, 2003, 2006, 2007 Free Software Foundation, Inc.
 
    This file is part of gnulib.
 
@@ -28,7 +28,6 @@
 #include <sys/socket.h>
 #include <sys/select.h>
 #include <unistd.h>
-#include <string.h>
 
 #ifdef HAVE_SYS_IOCTL_H
 #include <sys/ioctl.h>
@@ -37,16 +36,8 @@
 #include <sys/filio.h>
 #endif
 
-#if TIME_WITH_SYS_TIME
-# include <sys/time.h>
-# include <time.h>
-#else
-# if HAVE_SYS_TIME_H
-#  include <sys/time.h>
-# else
-#  include <time.h>
-# endif
-#endif
+#include <sys/time.h>
+#include <time.h>
 
 #ifndef INFTIM
 #define INFTIM (-1)
@@ -64,7 +55,7 @@ poll (pfd, nfd, timeout)
 {
   fd_set rfds, wfds, efds;
   struct timeval tv, *ptv;
-  int maxfd, rc, happened;
+  int maxfd, rc;
   nfds_t i;
 
 #ifdef _SC_OPEN_MAX
@@ -134,70 +125,77 @@ poll (pfd, nfd, timeout)
 			       | POLLWRNORM | POLLWRBAND)))
 	{
 	  maxfd = pfd[i].fd;
+
+	  /* Windows use a linear array of sockets (of size FD_SETSIZE). The
+	     descriptor value is not used to address the array.  */ 
+#if defined __CYGWIN__ || (!defined _WIN32 && !defined __WIN32__)
 	  if (maxfd > FD_SETSIZE)
 	    {
 	      errno = EOVERFLOW;
 	      return -1;
 	    }
+#endif
 	}
     }
 
   /* examine fd sets */
   rc = select (maxfd + 1, &rfds, &wfds, &efds, ptv);
+  if (rc < 0)
+    return rc;
 
   /* establish results */
-  if (rc > 0)
-    {
-      rc = 0;
-      for (i = 0; i < nfd; i++)
-	{
-	  pfd[i].revents = 0;
-	  if (pfd[i].fd < 0)
-	    continue;
-
-	  happened = 0;
-	  if (FD_ISSET (pfd[i].fd, &rfds))
-	    {
-	      int r;
+  rc = 0;
+  for (i = 0; i < nfd; i++)
+    if (pfd[i].fd < 0)
+      pfd[i].revents = 0;
+    else
+      {
+	int happened = 0, sought = pfd[i].events;
+	if (FD_ISSET (pfd[i].fd, &rfds))
+	  {
+	    int r;
 
 #if defined __MACH__ && defined __APPLE__
-	      /* There is a bug in Mac OS X that causes it to ignore MSG_PEEK for
-	         some kinds of descriptors. Use a length of 0. */
-	      r = recv (pfd[i].fd, NULL, 0, MSG_PEEK);
-	      if (r == 0)
-		happened = POLLIN | POLLRDNORM;
+	    /* There is a bug in Mac OS X that causes it to ignore MSG_PEEK
+	       for some kinds of descriptors.  Detect if this descriptor is a
+	       connected socket, a server socket, or something else using a
+	       0-byte recv, and use ioctl(2) to detect POLLHUP.  */
+	    r = recv (pfd[i].fd, NULL, 0, MSG_PEEK);
+	    if (r == 0 || errno == ENOTSOCK)
+	      ioctl(pfd[i].fd, FIONREAD, &r);
 #else
-	      char data[64];
-
-	      r = recv (pfd[i].fd, data, sizeof (data), MSG_PEEK);
-	      if (r == 0)
-		happened = POLLHUP;
+	    char data[64];
+	    r = recv (pfd[i].fd, data, sizeof (data), MSG_PEEK);
 #endif
-	      else if (r > 0)
-		happened = POLLIN | POLLRDNORM;
+	    if (r == 0)
+	      happened |= POLLHUP;
 
-	      else if (r < 0)
-		{
-		  if (errno == ENOTCONN)
-		    happened = POLLIN | POLLRDNORM;	/* Event happening on an unconnected server socket. */
-		  else
-		    happened = (errno == ESHUTDOWN || errno == ECONNRESET ||
-				errno == ECONNABORTED
-				|| errno == ENETRESET) ? POLLHUP : POLLERR;
-		  errno = 0;
-		}
-	    }
+	    /* If the event happened on an unconnected server socket,
+	       that's fine. */
+	    else if (r > 0 || ( /* (r == -1) && */ errno == ENOTCONN))
+	      happened |= (POLLIN | POLLRDNORM) & sought;
 
-	  if (FD_ISSET (pfd[i].fd, &wfds))
-	    happened |= POLLOUT | POLLWRNORM | POLLWRBAND;
+	    /* Distinguish hung-up sockets from other errors.  */
+	    else if (errno == ESHUTDOWN || errno == ECONNRESET
+		     || errno == ECONNABORTED || errno == ENETRESET)
+	      happened |= POLLHUP;
 
-	  if (FD_ISSET (pfd[i].fd, &efds))
-	    happened |= POLLPRI | POLLRDBAND;
+	    else
+	      happened |= POLLERR;
+	  }
 
-	  pfd[i].revents |= happened;
-	  rc += (happened > 0);
-	}
-    }
+	if (FD_ISSET (pfd[i].fd, &wfds))
+	  happened |= (POLLOUT | POLLWRNORM | POLLWRBAND) & sought;
+
+	if (FD_ISSET (pfd[i].fd, &efds))
+	  happened |= (POLLPRI | POLLRDBAND) & sought;
+
+	if (happened)
+	  {
+	    pfd[i].revents = happened;
+	    rc++;
+	  }
+      }
 
   return rc;
 }
